/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Originally written by Alphex Kaanoken <kaanoken@crew.org.ru>
 *
 *  (c) Crew IT Research Labs 2004
 *
 */

#include "detection.h"
#include "twolpoint.h"

/*this functions determine the symbol (a|b)*/
/*NOTE: This function generated automatically by some my mathematical model*/
int isAB(char byte)
{
  switch(byte)
    {
    case 'a':
      return 1;
      break;
    case 'b':
      return 0;
      break;
    case 'c':
      return 0;
      break;
    case 'd':
      return 0;
      break;
    case 'e':
      return 1;
      break;
    case 'f':
      return 0;
      break;
    case 'g':
      return 0;
      break;
    case 'h':
      return 0;
      break;
    case 'i':
      return 1;
      break;
    case 'j':
      return 0;
      break;
    case 'k':
      return 0;
      break;
    case 'l':
      return 0;
      break;
    case 'm':
      return 0;
      break;
    case 'n':
      return 0;
      break;
    case 'o':
      return 1;
      break;
    case 'p':
      return 0;
      break;
    case 'q':
      return 0;
      break;
    case 'r':
      return 0;
      break;
    case 's':
      return 0;
      break;
    case 't':
      return 0;
      break;
    case 'u':
      return 1;
      break;
    case 'v':
      return 0;
      break;
    case 'w':
      return 0;
      break;
    case 'x':
      return 0;
      break;
    case 'y':
      return 1;
      break;
    case 'z':
      return 0;
      break;
    default:
      return -1;
    }
}

/*this function determine this is Cyrillic word or not?
 * return percentage of this. (analysis of symbols)
 * Some part of this functions was generated automatically, 
 * and code looks not beatyfully, but it just works good and faster - it was 
 * was my target
 */
int isCyrWord(const char *word)
{
  int i=0;
  int lw=strlen(word);
  int points=0;
  int tk=0;
  int shr=0;
  if(lw<=1)
    return 0; 
  if(lw==2) /*simple 2 symbol word analysis*/
    {
      /*TODO: make a normal analysis with small words*/
      i=isAB(word[0])+isAB(word[1]);
      if(i==0)
	return -1;
      if(i<0)
	return 100;
    }
  /*detect for steps*/
  tk=lw/4;
  shr=lw%4;
  i=-1;
  if(tk>=1) {
    while(i!=shr) {
      if(!isAB(word[i+1])&&!isAB(word[i+2])&&!isAB(word[i+3])&&!isAB(word[i+4]))
	points+=60;
      if(isAB(word[i+1])&&isAB(word[i+2])&&isAB(word[i+3])&&isAB(word[i+4]))
	points+=60;
      ++i;
    }
  }
  /*stage one - we see for 4a|b in word, with moving*/
  i=0;
  while(i!=lw-1)
    {
      switch(word[i]){
      case '[':
	points+=80;
	break;
      case ']':
	points+=80;
	break;
      case '\'':
	points+=80;
	break;
      case ',':
	points+=80;
	break;
      case '.':
	points+=80;
	break;
      case ';':
	points+=80;
	break;
      default:
	break;
      }
      ++i;
    }
  /*stage two - we see for {}[];:'",.<> in word*/
  /*stage three lexical analysis
   * It's point was taken from output of my algorythm of searching for
   * slogs in english and russian. I'm use two dictionaryes
   */
  /*NOTE: this part generated automatically by my model, in future this model
   will be stored in CVS repositary and I add support for other languages 
  dynamically*/
  tk=lw/2;
  shr=lw%2;
  i=-1;
  if(tk>=1) {
    while(i!=shr) {
      switch(word[i+1]) {
	/*check 2l B*/
      case 'b':
	switch(word[i+2]){
	case 'c':
	  points+=BC;
	  break;
	case 'd':
	  points+=BD;
	  break;
	case 'f':
	  points+=BF;
	  break;
	case 'g':
	  points+=BG;
	  break;
	case 'h':
	  points+=BH;
	  break;
	case 'j':
	  points+=BJ;
	  break;
	case 'k':
	  points+=BK;
	  break;
	case 'n':
	  points+=BN;
	  break;
	case 'p':
	  points+=BP;
	  break;
	case 'q':
	  points+=BQ;
	  break;
	case 't':
	  points+=BT;
	  break;
	case 'v':
	  points+=BV;
	  break;
	case 'w':
	  points+=BW;
	  break;
	}
	break;
	/*check 2l C*/
      case 'c':
	switch(word[i+2]){
	case 'b':
	  points+=CB;
	  break;
	case 'd':
	  points+=CD;
	  break;
	case 'f':
	  points+=CF;
	  break;
	case 'g':
	  points+=CG;
	  break;
	case 'j':
	  points+=CJ;
	  break;
	case 'm':
	  points+=CM;
	  break;
	case 'n':
	  points+=CN;
	  break;
	case 'v':
	  points+=CV;
	  break;
	case 'w':
	  points+=CW;
	  break;
	case 'x':
	  points+=CX;
	  break;
	case 'z':
	  points+=CZ;
	  break;
	}
	break;
	/*check 2l D*/
      case 'd':
	switch(word[i+2]){
	case 'b':
	  points+=DB;
	  break;
	case 'c':
	  points+=DC;
	  break;
	case 'f':
	  points+=DF;
	  break;
	case 'h':
	  points+=DH;
	  break;
	case 'j':
	  points+=DJ;
	  break;
	case 'k':
	  points+=DK;
	  break;
	case 'n':
	  points+=DN;
	  break;
	case 'p':
	  points+=DP;
	  break;
	case 's':
	  points+=DS;
	  break;
	case 't':
	  points+=DT;
	  break;
	case 'x':
	  points+=DX;
	  break;
	case 'z':
	  points+=DZ;
	  break;
	}
	break;
	/*check 2l E*/
      case 'e':
	switch(word[i+2]){
	case 'h':
	  points+=EH;
	  break;
	case 'k':
	  points+=EK;
	  break;
	}
	break;
	/*check 2l F*/
      case 'f':
	switch(word[i+2]){
	case 'b':
	  points+=FB;
	  break;
	case 'c':
	  points+=FC;
	  break;
	case 'd':
	  points+=FD;
	  break;
	case 'g':
	  points+=FG;
	  break;
	case 'h':
	  points+=FH;
	  break;
	case 'j':
	  points+=FJ;
	  break;
	case 'k':
	  points+=FK;
	  break;
	case 'n':
	  points+=FN;
	  break;
	case 'p':
	  points+=FP;
	  break;
	case 'q':
	  points+=FQ;
	  break;
	case 'v':
	  points+=FV;
	  break;
	case 'w':
	  points+=FW;
	  break;
	case 'x':
	  points+=FX;
	  break;
	case 'y':
	  points+=FY;
	  break;
	case 'z':
	  points+=FZ;
	  break;
	}
	break;
	/*check 2l G*/
      case 'g':
	switch(word[i+2]){
	case 'b':
	  points+=GB;
	  break;
	case 'j':
	  points+=GJ;
	  break;
	case 'k':
	  points+=GK;
	  break;
	case 't':
	  points+=GT;
	  break;
	case 'x':
	  points+=GX;
	  break;
	case 'z':
	  points+=GZ;
	  break;
	}
	break;
	/*check 2l H*/
      case 'h':
	switch(word[i+2]){
	case 'b':
	  points+=HB;
	  break;
	case 'c':
	  points+=HC;
	  break;
	case 'd':
	  points+=HD;
	  break;
	case 'f':
	  points+=HF;
	  break;
	case 'g':
	  points+=HG;
	  break;
	case 'j':
	  points+=HJ;
	  break;
	case 'k':
	  points+=HK;
	  break;
	case 'l':
	  points+=HL;
	  break;
	case 'm':
	  points+=HM;
	  break;
	case 'n':
	  points+=HN;
	  break;
	case 's':
	  points+=HS;
	  break;
	case 'v':
	  points+=HV;
	  break;
	case 'x':
	  points+=HX;
	  break;
	case 'z':
	  points+=_HZ;
	  break;
	}
	break;
	/*check 2l J*/
      case 'j':
	switch(word[i+2]){
	case 'b':
	  points+=JB;
	  break;
	case 'c':
	  points+=JC;
	  break;
	case 'd':
	  points+=JD;
	  break;
	case 'f':
	  points+=JF;
	  break;
	case 'g':
	  points+=JG;
	  break;
	case 'h':
	  points+=JH;
	  break;
	case 'i':
	  points+=JI;
	  break;
	case 'j':
	  points+=JJ;
	  break;
	case 'k':
	  points+=JK;
	  break;
	case 'l':
	  points+=JL;
	  break;
	case 'n':
	  points+=JN;
	  break;
	case 'p':
	  points+=JP;
	  break;
	case 'q':
	  points+=JQ;
	  break;
	case 'r':
	  points+=JR;
	  break;
	case 't':
	  points+=JT;
	  break;
	case 'v':
	  points+=JV;
	  break;
	case 'w':
	  points+=JW;
	  break;
	case 'x':
	  points+=JX;
	  break;
	case 'y':
	  points+=JY;
	  break;
	case 'z':
	  points+=JZ;
	  break;
	}
	break;
	/*check 2l K*/
      case 'k':
	switch(word[i+2]){
	case 'b':
	  points+=KB;
	  break;
	case 'f':
	  points+=KF;
	  break;
	case 'j':
	  points+=KJ;
	  break;
	case 'k':
	  points+=KK;
	  break;
	case 'm':
	  points+=KM;
	  break;
	case 't':
	  points+=KT;
	  break;
	case 'v':
	  points+=KV;
	  break;
	case 'x':
	  points+=KX;
	  break;
	case 'z':
	  points+=KZ;
	  break;
	}
	break;
      case 'l':
	switch(word[i+2]){
	case 'j':
	  points+=LJ;
	  break;
	case 'x':
	  points+=LX;
	  break;
	case 'z':
	  points+=LZ;
	  break;
	}
	break;
	/*check 2l M*/
      case 'm':
	switch(word[i+2]){
	case 'c':
	  points+=MC;
	  break;
	case 'd':
	  points+=MD;
	  break;
	case 'v':
	  points+=MV;
	  break;
	case 'x':
	  points+=MX;
	  break;
	case 'z':
	  points+=MZ;
	  break;
	}
	break;
	/*check 2l N*/
      case 'n':
	switch(word[i+2]){
	case 'j':
	  points+=NJ;
	  break;
	case 'm':
	  points+=NM;
	  break;
	case 'z':
	  points+=NZ;
	  break;
	}
	break;
	/*check 2l P*/
      case 'p':
	switch(word[i+2]){
	case 'b':
	  points+=PB;
	  break;
	case 'd':
	  points+=PD;
	  break;
	case 'f':
	  points+=PF;
	  break;
	case 'j':
	  points+=PJ;
	  break;
	case 'k':
	  points+=PK;
	  break;
	case 'v':
	  points+=PV;
	  break;
	case 'x':
	  points+=PX;
	  break;
	case 'z':
	  points+=PZ;
	  break;
	}
	break;
	/*check 2l Q*/
      case 'q':
	switch(word[i+2]){
	case 'a':
	  points+=QA;
	  break;
	case 'c':
	  points+=QC;
	  break;
	case 'd':
	  points+=QD;
	  break;
	case 'g':
	  points+=QG;
	  break;
	case 'i':
	  points+=QI;
	  break;
	case 'j':
	  points+=QJ;
	  break;
	case 'k':
	  points+=QK;
	  break;
	case 'l':
	  points+=QL;
	  break;
	case 'n':
	  points+=QN;
	  break;
	case 'o':
	  points+=QO;
	  break;
	case 'r':
	  points+=QR;
	  break;
	case 'v':
	  points+=QV;
	  break;
	case 'w':
	  points+=QW;
	  break;
	case 'x':
	  points+=QX;
	  break;
	case 'y':
	  points+=QY;
	  break;
	case 'z':
	  points+=QZ;
	  break;
	}
	break;
	/*check 2l R*/
      case 'r':
	switch(word[i+2]){
	case 'f':
	  points+=RF;
	  break;
	case 'h':
	  points+=RH;
	  break;
	case 'j':
	  points+=RJ;
	  break;
	}
	break;
	/*check 2l S*/
      case 's':
	switch(word[i+2]){
	case 'd':
	  points+=SD;
	  break;
	case 'q':
	  points+=SQ;
	  break;
	case 'x':
	  points+=SX;
	  break;
	case 'z':
	  points+=SZ;
	  break;
	}
	break;
      case 't':
	switch(word[i+2]){
	case 'c':
	  points+=TC;
	  break;
	case 'd':
	  points+=TD;
	  break;
	case 'g':
	  points+=TG;
	  break;
	case 'j':
	  points+=TJ;
	  break;
	case 'k':
	  points+=TK;
	  break;
	case 'n':
	  points+=TN;
	  break;
	case 'p':
	  points+=TP;
	  break;
	case 'q':
	  points+=TQ;
	  break;
	case 'v':
	  points+=TV;
	  break;
	case 'x':
	  points+=TX;
	  break;
	}
	break;
	/*check 2l U*/
      case 'u':
	switch(word[i+2]){
	case 'h':
	  points+=UH;
	  break;
	case 'j':
	  points+=UJ;
	  break;
	case 'k':
	  points+=UK;
	  break;
	}
	break;
	/*check 2l V*/
      case 'v':
	switch(word[i+2]){
	case 'b':
	  points+=VB;
	  break;
	case 'c':
	  points+=VC;
	  break;
	case 'd':
	  points+=VD;
	  break;
	case 'f':
	  points+=VF;
	  break;
	case 'g':
	  points+=VG;
	  break;
	case 'h':
	  points+=VH;
	  break;
	case 'j':
	  points+=VJ;
	  break;
	case 'k':
	  points+=VK;
	  break;
	case 'r':
	  points+=VR;
	  break;
	case 's':
	  points+=VS;
	  break;
	case 't':
	  points+=VT;
	  break;
	case 'z':
	  points+=VZ;
	  break;
	}
	break;
	/*check 2l W*/
      case 'w':
	switch(word[i+2]){
	case 'b':
	  points+=WB;
	  break;
	case 'j':
	  points+=WJ;
	  break;
	case 't':
	  points+=WT;
	  break;
	}
	break;
	/*check 2l X*/
      case 'x':
	switch(word[i+2]){
	case 'b':
	  points+=XB;
	  break;
	case 'f':
	  points+=XF;
	  break;
	case 'j':
	  points+=XJ;
	  break;
	case 'k':
	  points+=XK;
	  break;
	case 'm':
	  points+=XM;
	  break;
	case 'n':
	  points+=XN;
	  break;
	case 'r':
	  points+=XR;
	  break;
	case 't':
	  points+=XT;
	  break;
	case 'v':
	  points+=XV;
	  break;
	case 'y':
	  points+=XY;
	  break;
	}
	break;
	/*check 2l Y*/
      case 'y':
	switch(word[i+2]){
	case 'b':
	  points+=YB;
	  break;
	case 'f':
	  points+=YF;
	  break;
	case 'j':
	  points+=YJ;
	  break;
	case 't':
	  points+=YT;
	  break;
	case 'u':
	  points+=YU;
	  break;
	case 'y':
	  points+=YY;
	  break;
	case 'z':
	  points+=YZ;
	  break;
	}
	break;
	/*check 2l Z*/
      case 'z':
	switch(word[i+2]){
	case 'c':
	  points+=ZC;
	  break;
	case 'd':
	  points+=ZD;
	  break;
	case 'h':
	  points+=ZH;
	  break;
	case 'n':
	  points+=ZN;
	  break;
	case 'p':
	  points+=ZP;
	  break;
	case 'q':
	  points+=ZQ;
	  break;
	case 'r':
	  points+=ZR;
	  break;
	case 't':
	  points+=ZT;
	  break;
	case 'u':
	  points+=ZU;
	  break;
	case 'v':
	  points+=ZV;
	  break;
	case 'w':
	  points+=ZW;
	  break;
	case 'x':
	  points+=ZX;
	  break;
	case 'y':
	  points+=ZY;
	  break;
	}
	break;
      }
      ++i;
    }
  }
  return points;
}

